/**
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

#include <XBee.h>

/**
* Código com a programação dos kits Arduino pertencentes ao DroidLar.
* 
* Esse código contém a lógica para o recebimento das mensagens provenientes 
* do SAR e a lógica para o envio de uma resposta ou a execução de alguma ação, 
* dependendo do tipo de mensagem recebida.
*
* Nesse código também há a lógica para a mudança de estado do dispositivo 
* quando o botão que o controla for pressionado.
*/

// Saída pro eletrodoméstico controlado (pino do led)
int DISP = 9;

// Estado da saída
int DISP_ESTADO = 0;

// Tipo de dispositivo controlado
int DISP_TIPO = 100;

// Variáveis para o controle do botao
int ButtonPin = 13;
int ButtonState = 0;
int LastState = 0;

// Mensagens do protocolo
static int MSG_CONTROLE = 3;
static int REQUEST_INFO = 8;
static int RESPONSE_INFO = 9;
static int UPDATE_ESTADO = 10;

// Objeto usado para a manipulação dos módulos XBee
XBee xbee = XBee();

// Objeto usado para acessar as mensagens recebidas
ZBRxResponse rx = ZBRxResponse();

// Informação contida no pacote de resposta
uint8_t payload[] = { 0, 0, 0 };

// SH + SL Address do XBee receptor
// SH = 0 e SL = 0 correspondem ao coordenador ZB
XBeeAddress64 addr64 = XBeeAddress64(0, 0);

// Objeto contendo o pacote a ser enviado
ZBTxRequest zbTx = ZBTxRequest(addr64, payload, sizeof(payload));

void setup() {
  
  // Inicia a serial
  xbee.begin(9600);
  
  delay(7000);
  
  // Envia mensagem ao SAR com suas informações
  executarMensagem();
  
  // Configura a saída pro dispositivo e configura 
  // ela com o valor inicial
  pinMode(ButtonPin, INPUT);
  pinMode(DISP, OUTPUT);
  analogWrite(DISP,DISP_ESTADO);
  
}

// Lê pacotes continuamente esperando mensagens do SAR e 
// também verifica se o botão foi pressionado
void loop() {
    
    xbee.readPacket();
    
    // Verifica se chegou uma mensagem
    if (xbee.getResponse().isAvailable()) {
      
      // Verifica se a mensagem foi enviada pelo SAR
      if (xbee.getResponse().getApiId() == ZB_RX_RESPONSE) {
        
            // Configura o objeto 'rx' com a mensagem recebida
            xbee.getResponse().getZBRxResponse(rx);
            
            // Indica o tipo de mensagem
            int tipo = rx.getData(0);
            // Mensagem recebida
            int mensagem = rx.getData(1);
            // O campo 'opcao' é variável, nesse caso ele nao esta sendo usado
            int opcao = rx.getData(2);

            if (tipo == REQUEST_INFO) {
                executarMensagem();
            } else if (tipo == MSG_CONTROLE) {
                DISP_ESTADO = mensagem;
                analogWrite(DISP,DISP_ESTADO);
            }
        
      }
    }

    // Verifica se o botao foi clicado    
    verificaBotoes();
   
}

// Envia uma resposta ao SAR de acordo a mensagem recebida
void executarMensagem(){      
        
  // Envia resposta ao SAR com as informações do dispositivo
         
  payload[0] = RESPONSE_INFO;         
                
  payload[1] = DISP_TIPO;
  payload[2] = DISP_ESTADO;
  // transmite a mensagem
  xbee.send(zbTx);
         
}

// Verifica se o estado da entrada digital, onde o botao esta conectados, foi alterado,
// o que indicaria que ele foi clicado
void verificaBotoes(){ 
    
    ButtonState = digitalRead(ButtonPin);
  
    if(ButtonState != LastState){
        if(ButtonState == HIGH){   
      
          if(DISP_ESTADO > 0){
              DISP_ESTADO = 0;
          }else{
              DISP_ESTADO = 255;
          }
          
          analogWrite(DISP, DISP_ESTADO);
          payload[0] = UPDATE_ESTADO;
          payload[1] = DISP_ESTADO;
          payload[2] = 0;
          // transmite a mensagem
    	  xbee.send(zbTx);
        }
        LastState = ButtonState;
    }
    delay(10);
  
}
